@@ -15,15 +15,12 @@
 #ifndef LLVM_CLANG_BASIC_MODULE_H
 #define LLVM_CLANG_BASIC_MODULE_H
 
-#include "clang/Basic/FileManager.h"
 #include "clang/Basic/SourceLocation.h"
-#include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/DenseSet.h"
 #include "llvm/ADT/PointerIntPair.h"
 #include "llvm/ADT/PointerUnion.h"
 #include "llvm/ADT/SetVector.h"
 #include "llvm/ADT/SmallVector.h"
-#include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/StringMap.h"
 #include "llvm/ADT/StringRef.h"
 #include <string>
@@ -36,23 +33,16 @@ namespace llvm {
 
 namespace clang {
   
+class DirectoryEntry;
+class FileEntry;
+class FileManager;
 class LangOptions;
 class TargetInfo;
 class IdentifierInfo;
   
 /// \brief Describes the name of a module.
 typedef SmallVector<std::pair<std::string, SourceLocation>, 2> ModuleId;
-
-/// The signature of a module, which is a hash of the AST content.
-struct ASTFileSignature : std::array<uint32_t, 5> {
-  ASTFileSignature(std::array<uint32_t, 5> S = {{0}})
-      : std::array<uint32_t, 5>(std::move(S)) {}
-
-  explicit operator bool() const {
-    return *this != std::array<uint32_t, 5>({{0}});
-  }
-};
-
+  
 /// \brief Describes a module or submodule.
 class Module {
 public:
@@ -62,39 +52,23 @@ public:
   /// \brief The location of the module definition.
   SourceLocation DefinitionLoc;
 
-  enum ModuleKind {
-    /// \brief This is a module that was defined by a module map and built out
-    /// of header files.
-    ModuleMapModule,
-
-    /// \brief This is a C++ Modules TS module interface unit.
-    ModuleInterfaceUnit
-  };
-
-  /// \brief The kind of this module.
-  ModuleKind Kind = ModuleMapModule;
-
   /// \brief The parent of this module. This will be NULL for the top-level
   /// module.
   Module *Parent;
 
-  /// \brief The build directory of this module. This is the directory in
-  /// which the module is notionally built, and relative to which its headers
-  /// are found.
-  const DirectoryEntry *Directory;
-
-  /// \brief The presumed file name for the module map defining this module.
-  /// Only non-empty when building from preprocessed source.
-  std::string PresumedModuleMapFile;
-
+  /// \brief The module map file that (along with the module name) uniquely
+  /// identifies this module.
+  ///
+  /// The particular module that \c Name refers to may depend on how the module
+  /// was found in header search. However, the combination of \c Name and
+  /// \c ModuleMap will be globally unique for top-level modules. In the case of
+  /// inferred modules, \c ModuleMap will contain the module map that allowed
+  /// the inference (e.g. contained 'Module *') rather than the virtual
+  /// inferred module map file.
+  const FileEntry *ModuleMap;
+  
   /// \brief The umbrella header or directory.
   llvm::PointerUnion<const DirectoryEntry *, const FileEntry *> Umbrella;
-
-  /// \brief The module signature.
-  ASTFileSignature Signature;
-
-  /// \brief The name of the umbrella entry, as written in the module map.
-  std::string UmbrellaAsWritten;
   
 private:
   /// \brief The submodules of this module, indexed by name.
@@ -117,59 +91,27 @@ private:
   /// \brief Cache of modules visible to lookup in this module.
   mutable llvm::DenseSet<const Module*> VisibleModulesCache;
 
-  /// The ID used when referencing this module within a VisibleModuleSet.
-  unsigned VisibilityID;
-
 public:
-  enum HeaderKind {
-    HK_Normal,
-    HK_Textual,
-    HK_Private,
-    HK_PrivateTextual,
-    HK_Excluded
-  };
-  static const int NumHeaderKinds = HK_Excluded + 1;
+  /// \brief The headers that are part of this module.
+  SmallVector<const FileEntry *, 2> NormalHeaders;
+
+  /// \brief The headers that are explicitly excluded from this module.
+  SmallVector<const FileEntry *, 2> ExcludedHeaders;
+
+  /// \brief The headers that are private to this module.
+  SmallVector<const FileEntry *, 2> PrivateHeaders;
 
   /// \brief Information about a header directive as found in the module map
   /// file.
-  struct Header {
-    std::string NameAsWritten;
-    const FileEntry *Entry;
-
-    explicit operator bool() { return Entry; }
-  };
-
-  /// \brief Information about a directory name as found in the module map
-  /// file.
-  struct DirectoryName {
-    std::string NameAsWritten;
-    const DirectoryEntry *Entry;
-
-    explicit operator bool() { return Entry; }
-  };
-
-  /// \brief The headers that are part of this module.
-  SmallVector<Header, 2> Headers[5];
-
-  /// \brief Stored information about a header directive that was found in the
-  /// module map file but has not been resolved to a file.
-  struct UnresolvedHeaderDirective {
-    HeaderKind Kind = HK_Normal;
+  struct HeaderDirective {
     SourceLocation FileNameLoc;
     std::string FileName;
-    bool IsUmbrella = false;
-    bool HasBuiltinHeader = false;
-    Optional<off_t> Size;
-    Optional<time_t> ModTime;
+    bool IsUmbrella;
   };
 
-  /// Headers that are mentioned in the module map file but that we have not
-  /// yet attempted to resolve to a file on the file system.
-  SmallVector<UnresolvedHeaderDirective, 1> UnresolvedHeaders;
-
   /// \brief Headers that are mentioned in the module map file but could not be
   /// found on the file system.
-  SmallVector<UnresolvedHeaderDirective, 1> MissingHeaders;
+  SmallVector<HeaderDirective, 1> MissingHeaders;
 
   /// \brief An individual requirement: a feature name and a flag indicating
   /// the required state of that feature.
@@ -184,9 +126,6 @@ public:
   /// \brief Whether this module is missing a feature from \c Requirements.
   unsigned IsMissingRequirement : 1;
 
-  /// \brief Whether we tried and failed to load a module file for this module.
-  unsigned HasIncompatibleModuleFile : 1;
-
   /// \brief Whether this module is available in the current translation unit.
   ///
   /// If the module is missing headers or does not meet all requirements then
@@ -235,15 +174,14 @@ public:
   /// built.
   unsigned ConfigMacrosExhaustive : 1;
 
-  /// \brief Whether files in this module can only include non-modular headers
-  /// and headers from used modules.
-  unsigned NoUndeclaredIncludes : 1;
-
   /// \brief Describes the visibility of the various names within a
   /// particular module.
   enum NameVisibilityKind {
     /// \brief All of the names in this module are hidden.
+    ///
     Hidden,
+    /// \brief Only the macro names in this module are visible.
+    MacrosVisible,
     /// \brief All of the names in this module are visible.
     AllVisible
   };
@@ -251,12 +189,15 @@ public:
   /// \brief The visibility of names within this particular module.
   NameVisibilityKind NameVisibility;
 
+  /// \brief The location at which macros within this module became visible.
+  SourceLocation MacroVisibilityLoc;
+
   /// \brief The location of the inferred submodule.
   SourceLocation InferredSubmoduleLoc;
 
   /// \brief The set of modules imported by this module, and on which this
   /// module depends.
-  llvm::SmallSetVector<Module *, 2> Imports;
+  SmallVector<Module *, 2> Imports;
   
   /// \brief Describes an exported module.
   ///
@@ -342,8 +283,10 @@ public:
   std::vector<Conflict> Conflicts;
 
   /// \brief Construct a new module or submodule.
+  ///
+  /// For an explanation of \p ModuleMap, see Module::ModuleMap.
   Module(StringRef Name, SourceLocation DefinitionLoc, Module *Parent,
-         bool IsFramework, bool IsExplicit, unsigned VisibilityID);
+         const FileEntry *ModuleMap, bool IsFramework, bool IsExplicit);
   
   ~Module();
   
@@ -365,7 +308,7 @@ public:
   bool isAvailable(const LangOptions &LangOpts, 
                    const TargetInfo &Target,
                    Requirement &Req,
-                   UnresolvedHeaderDirective &MissingHeader) const;
+                   HeaderDirective &MissingHeader) const;
 
   /// \brief Determine whether this module is a submodule.
   bool isSubModule() const { return Parent != nullptr; }
@@ -393,15 +336,7 @@ public:
 
   /// \brief Retrieve the full name of this module, including the path from
   /// its top-level module.
-  /// \param AllowStringLiterals If \c true, components that might not be
-  ///        lexically valid as identifiers will be emitted as string literals.
-  std::string getFullModuleName(bool AllowStringLiterals = false) const;
-
-  /// \brief Whether the full name of this module is equal to joining
-  /// \p nameParts with "."s.
-  ///
-  /// This is more efficient than getFullModuleName().
-  bool fullModuleNameIs(ArrayRef<StringRef> nameParts) const;
+  std::string getFullModuleName() const;
 
   /// \brief Retrieve the top-level module for this (sub)module, which may
   /// be this module.
@@ -434,14 +369,12 @@ public:
 
   /// \brief Retrieve the directory for which this module serves as the
   /// umbrella.
-  DirectoryName getUmbrellaDir() const;
+  const DirectoryEntry *getUmbrellaDir() const;
 
   /// \brief Retrieve the header that serves as the umbrella header for this
   /// module.
-  Header getUmbrellaHeader() const {
-    if (auto *E = Umbrella.dyn_cast<const FileEntry *>())
-      return Header{UmbrellaAsWritten, E};
-    return Header{};
+  const FileEntry *getUmbrellaHeader() const {
+    return Umbrella.dyn_cast<const FileEntry *>();
   }
 
   /// \brief Determine whether this module has an umbrella directory that is
@@ -464,10 +397,6 @@ public:
   /// \brief The top-level headers associated with this module.
   ArrayRef<const FileEntry *> getTopHeaders(FileManager &FileMgr);
 
-  /// \brief Determine whether this module has declared its intention to
-  /// directly use another module.
-  bool directlyUses(const Module *Requested) const;
-
   /// \brief Add the given feature requirement to the list of features
   /// required by this module.
   ///
@@ -506,8 +435,6 @@ public:
     return VisibleModulesCache.count(M);
   }
 
-  unsigned getVisibilityID() const { return VisibilityID; }
-
   typedef std::vector<Module *>::iterator submodule_iterator;
   typedef std::vector<Module *>::const_iterator submodule_const_iterator;
   
@@ -516,13 +443,6 @@ public:
   submodule_iterator submodule_end()   { return SubModules.end(); }
   submodule_const_iterator submodule_end() const { return SubModules.end(); }
 
-  llvm::iterator_range<submodule_iterator> submodules() {
-    return llvm::make_range(submodule_begin(), submodule_end());
-  }
-  llvm::iterator_range<submodule_const_iterator> submodules() const {
-    return llvm::make_range(submodule_begin(), submodule_end());
-  }
-
   /// \brief Appends this module's list of exported modules to \p Exported.
   ///
   /// This provides a subset of immediately imported modules (the ones that are
@@ -544,65 +464,6 @@ private:
   void buildVisibleModulesCache() const;
 };
 
-/// \brief A set of visible modules.
-class VisibleModuleSet {
-public:
-  VisibleModuleSet() : Generation(0) {}
-  VisibleModuleSet(VisibleModuleSet &&O)
-      : ImportLocs(std::move(O.ImportLocs)), Generation(O.Generation ? 1 : 0) {
-    O.ImportLocs.clear();
-    ++O.Generation;
-  }
-
-  /// Move from another visible modules set. Guaranteed to leave the source
-  /// empty and bump the generation on both.
-  VisibleModuleSet &operator=(VisibleModuleSet &&O) {
-    ImportLocs = std::move(O.ImportLocs);
-    O.ImportLocs.clear();
-    ++O.Generation;
-    ++Generation;
-    return *this;
-  }
-
-  /// \brief Get the current visibility generation. Incremented each time the
-  /// set of visible modules changes in any way.
-  unsigned getGeneration() const { return Generation; }
-
-  /// \brief Determine whether a module is visible.
-  bool isVisible(const Module *M) const {
-    return getImportLoc(M).isValid();
-  }
-
-  /// \brief Get the location at which the import of a module was triggered.
-  SourceLocation getImportLoc(const Module *M) const {
-    return M->getVisibilityID() < ImportLocs.size()
-               ? ImportLocs[M->getVisibilityID()]
-               : SourceLocation();
-  }
-
-  /// \brief A callback to call when a module is made visible (directly or
-  /// indirectly) by a call to \ref setVisible.
-  typedef llvm::function_ref<void(Module *M)> VisibleCallback;
-  /// \brief A callback to call when a module conflict is found. \p Path
-  /// consists of a sequence of modules from the conflicting module to the one
-  /// made visible, where each was exported by the next.
-  typedef llvm::function_ref<void(ArrayRef<Module *> Path,
-                                  Module *Conflict, StringRef Message)>
-      ConflictCallback;
-  /// \brief Make a specific module visible.
-  void setVisible(Module *M, SourceLocation Loc,
-                  VisibleCallback Vis = [](Module *) {},
-                  ConflictCallback Cb = [](ArrayRef<Module *>, Module *,
-                                           StringRef) {});
-
-private:
-  /// Import locations for each visible module. Indexed by the module's
-  /// VisibilityID.
-  std::vector<SourceLocation> ImportLocs;
-  /// Visibility generation, bumped every time the visibility state changes.
-  unsigned Generation;
-};
-
 } // end namespace clang
 
 
